@@ -41,14 +41,14 @@ use DBI;
 # We expect the non-null test to return row 3 (Marge)
 # and the null test to return rows 2 and 4 (the undefs).
 		
-my $homer = "Homer   ";
-my $marge = "Marge   ";
+my $homer = "Homer";
+my $marge = "Marge";
 
 my @char_column_values = (
-  $homer,
-  undef,
-  $marge,
-  undef
+  $homer,   # 1
+  undef,    # 2
+  $marge,   # 3
+  undef,    # 4
 );
 
 # Define the SQL statements with the various WHERE clause styles we want to test
@@ -89,16 +89,12 @@ my $sth;
 my @ok;
 
 print "=> Drop table '$tablename', if it already exists...\n";
-$sth = $dbh->do("DROP TABLE $tablename");
+do { local $dbh->{PrintError}=0; $dbh->do("DROP TABLE $tablename"); };
 
 print "=> Create table '$tablename'...\n";
-$sth = $dbh->prepare("CREATE TABLE $tablename (myid int NOT NULL, mycol char(8))");
-
-# Use this if your database does not support NULL columns by default.
-#$sth = $dbh->prepare("CREATE TABLE $tablename (myid int NOT NULL, mycol char(8) NULL)");
-
-$sth->execute()
- || $sth->errstr;
+$dbh->do("CREATE TABLE $tablename (myid int NOT NULL, mycol char(5))");
+# Use this if your database does not support NULL columns by default:
+#$dbh->do("CREATE TABLE $tablename (myid int NOT NULL, mycol char(5) NULL)");
 
 print "=> Insert 4 rows into the table...\n";
 
@@ -106,66 +102,75 @@ $sth = $dbh->prepare("INSERT INTO $tablename (myid, mycol) VALUES (?,?)");
 for my $i (0..$#char_column_values)
 {
     my $val = $char_column_values[$i];
-    printf "Values %d %s\n", $i+1, defined($val)? $val : "";
-    $sth->execute($i+1, $val)
-      || $sth->errstr;
+    printf "   Inserting values (%d, %s)\n", $i+1, $dbh->quote($val);
+    $sth->execute($i+1, $val);
 }
+print "(Driver bug: statement handle should not be Active after an INSERT.)\n"
+    if $sth->{Active};
+
 # Run the tests...
 
 for my $i (0..$#select_clauses)
 {
     my $sel = $select_clauses[$i];
-    print "\n=> Testing clause style $i: ".$sel->{clause}."\n";
+    print "\n=> Testing clause style $i: ".$sel->{clause}."...\n";
     
     $sth = $dbh->prepare("SELECT myid,mycol FROM $tablename ".$sel->{clause})
 	or next;
 
+    print "   Selecting row with $marge\n";
     $sth->execute(@{$sel->{nonnull}})
 	or next;
     my $r1 = $sth->fetchall_arrayref();
-    my $n1r = $sth->rows;
+    my $n1_rows = $sth->rows;
     my $n1 = @$r1;
     
+    print "   Selecting rows with NULL\n";
     $sth->execute(@{$sel->{null}})
 	or next;
     my $r2 = $sth->fetchall_arrayref();
-    my $n2r = $sth->rows;
+    my $n2_rows = $sth->rows;
     my $n2 = @$r2;
     
     # Complain a bit...
     
     print "\n=>Your DBD driver doesn't support the 'rows' method very well.\n\n"
-       unless ($n1r == $n1 && $n2r == $n2);
+       unless ($n1_rows == $n1 && $n2_rows == $n2);
        
     # Did we get back the expected "n"umber of rows?
     # Did we get back the specific "r"ows we expected as identifed by the myid column?
     
-    if (   $n1 == 1
-        && $n2 == 2
-        && $r1->[0][0] == 3
-        && $r2->[0][0] == 2
-        && $r2->[1][0] == 4)
-    {
+    if (   $n1 == 1     # one row for Marge
+        && $n2 == 2     # two rows for nulls
+        && $r1->[0][0] == 3 # Marge is myid 3
+        && $r2->[0][0] == 2 # NULL for myid 2
+        && $r2->[1][0] == 4 # NULL for myid 4
+    ) {
       print "=> WHERE clause style $i is supported.\n";
-      push @ok, "$i: ".$sel->{clause};
+      push @ok, "\tStyle $i: ".$sel->{clause};
     }
     else
     {
       print "=> WHERE clause style $i returned incorrect results.\n";
       if ($n1 > 0 || $n2 > 0)
       {
-        print "    Non-Null test rows returned: ";
-        print " ", $r1->[$_][0] for (0..$#{$r1});
-        print "\n";
-        print "    Null test rows returned: ";
-        print " ", $r2->[$_][0] for (0..$#{$r2});
-        print "\n";
+        print "   Non-NULL test rows returned these row ids: ".
+            join(", ", map { $r1->[$_][0] } (0..$#{$r1}))."\n";
+        print "   The NULL test rows returned these row ids: ".
+            join(", ", map { $r2->[$_][0] } (0..$#{$r2}))."\n";
       }
     }
 }
 
 $dbh->disconnect();
-
-printf "\n%d styles are supported\n", scalar @ok;
-print "$_\n" for @ok;
 print "\n";
+print "-" x 72, "\n";
+printf "%d styles are supported:\n", scalar @ok;
+print "$_\n" for @ok;
+print "-" x 72, "\n";
+print "\n";
+print "If these results don't match what's in the 'Placeholders and Bind Values'\n";
+print "section of the DBI documentation, or are for a database that not already\n";
+print "listed, please email the results to dbi-users\@perl.org. Thank you.\n";
+
+exit 0;
