@@ -20,7 +20,7 @@ sub test_dir
 	$test_dir = File::Spec->rel2abs( File::Spec->curdir () );
 	$test_dir = File::Spec->catdir ( $test_dir, "test_output_" . $$ );
 	$test_dir = VMS::Filespec::unixify($test_dir) if $^O eq 'VMS';
-	rmtree $test_dir;
+	rmtree $test_dir if -d $test_dir;
 	mkpath $test_dir;
 	# There must be at least one directory in the test directory,
 	# and nothing guarantees that dot or dot-dot directories will exist.
